/***********************************************************
 *                   FileManager                           *
 * This file contains JavaScript types and functions which *
 * provide information about local files and directories.  *
 *                                                         *
 * It is designed to work across all platforms and so      *
 * provides types to manipulate paths without entering     *
 * path delimiters.                                        *
 *                                                         *
 * There are types available to clearly distinguish        *
 * filenames which are specified by absolute path          *
 * form those specified by relative path.                  *
 *                                                         *
 * A filename can be presented in either the native        *
 * format or as a file: URI.                               *
 *                                                         *
 * T_Path - A base type which represents the path          *
 *          to a file.                                     *
 *                                                         *
 * T_File - Represents a local file.                       *
 *                                                         *
 *                                                         *
 * Uses                                                    *
 *   generalsupport.js                                     *
 *                                                         *
 *                      History                            *
 * 07/03/2006  Initial Version                             *
 * 13/04/2006  Implemented FileIO (Ref MozillaZine File IO)*
 * 23/05/2006  HotLine Version 4.0                         *
 * 29/05/2006  Added T_Directory                           *
 * 30/09/2009  Added T_LocalFile                           *
 ***********************************************************/


//**********************************************************
//                T_Path                                   *
// This is the base type for the intrinsic directory       *
// types defined below. The descendants create a new       *
// base type specifying the intrinsic directories defined  *
// by these constants.                                     *
// If a FileName is specified when the Path is created,    *
// it is appended to the Path, thus creating the Path      *
// as a File Path.                                         *
//**********************************************************

//  Intrinsic Directories Path

T_Path.CURRENT_PROFILE_DIRECTORYV     = "ProfD";
T_Path.DEFAULT_PROFILES_DIRECTORYV    = "DefProfRt";
T_Path.CHROME_DIRECTORYV              = "UChrm";
T_Path.TEMP_DIRECTORYV                = "TmpD";
T_Path.HOME_DIRECTORYV                = "Home";
T_Path.INSTALLATION_DIRECTORYV        = "CurProcD";
T_Path.PREFERENCES_DIRECTORYV         = "PrefD";
T_Path.EXTENSION_DIRECTORYV           = "";

//  Intrinsic Files Path

T_Path.PREFERENCES_FILEV              = "PrefF";

//  nsIFile Create 
T_Path.NORMAL_FILE_TYPEV              = 0;
T_Path.DIRECTORY_TYPEV                = 1;

//  T_Path Class and Properties

function T_Path(IntrinsicName){
  //  Save the IntrinsicName when the object is created,
  //  and only get the path in Initialize, since a new
  //  object is created as the prototype for descendant
  //  types.
  this.nsIFile = undefined;
  this.IntrinsicName = IntrinsicName;
}// of T_Path

//  T_Path Methods

T_Path.prototype.Initialize = function(FileName){
  try {
    this.nsIFile = Components.classes["@mozilla.org/file/directory_service;1"]
                                          .getService(Components.interfaces.nsIProperties)
                                                .get(this.IntrinsicName, Components.interfaces.nsIFile);
  }
  catch(e) {
    this.nsIFile = undefined;
  };
  this.IsValid = (this.nsIFile != undefined);
  //  If a FileName has been provided, then append it to
  //  the Directory.
  if ((this.IsValid) && (arguments.length > 0))
    this.nsIFile.append(FileName);
//  return (this.nsIFile == undefined ? Error(e) : OK());
}// of T_Path.prototype.Initialize

T_Path.prototype.ToDisplay = function(){
  //  Return a Display Form String of the Directory
  return this.nsIFile.path;
}// of T_Path.prototype.ToDisplay

T_Path.prototype.ToURI = function(){
  //  Return a URI Display Form String of the Directory
  var ioservice = Components.classes["@mozilla.org/network/io-service;1"]
                                          .getService(Components.interfaces.nsIIOService);
  var fileHandler = ioservice.getProtocolHandler("file")
                           .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
  return fileHandler.getURLSpecFromFile(this.nsIFile);
}// of T_Path.prototype.ToURI

T_Path.prototype.To_nsIURI = function(){
  //  Return a nsIURI object reference to the file.
  var ioservice = Components.classes["@mozilla.org/network/io-service;1"]
                                          .getService(Components.interfaces.nsIIOService);
  return ioservice.newFileURI(this.nsIFile);
}// of T_Path.prototype.To_nsIURI

T_Path.prototype.Append = function(FileName){
  //  Appends the FileName supplied to the current Path
  //  using the appropriate directory separator for
  //  this system.
  this.nsIFile.append(FileName);
  return this.nsIFile.path;
}// of T_Path.prototype.Append

T_Path.prototype.ItExists = function(){
  return this.nsIFile.exists();
}// of T_Path.prototype.ItExists

T_Path.prototype.Resident = function(){
  return this.nsIFile.exists();
}// of T_Path.prototype.Resident

T_Path.prototype.ItsADirectory = function(){
  return this.nsIFile.isDirectory();
}// of T_Path.prototype.ItsADirectory

T_Path.prototype.LastNodeName = function(){
  return this.nsIFile.leafName;
}// of T_Path.prototype.LastNodeName

T_Path.prototype.ParentFileName = function(){
  return this.nsIFile.parent.path;
}// of T_Path.prototype.ParentFileName

T_Path.prototype.Clone = function(){
  var Path = new this.constructor();
  Path.nsIFile=undefined;
  Path.IntrinsicName=this.IntrinsicName;
  if (this.nsIFile)
    Path.nsIFile=this.nsIFile.clone();
  Path.IsValid=(Path.nsIFile != undefined);
  return Path;
}// of T_Path.prototype.Clone

//  T_Path Class Items

//**********************************************************
//                T_ProfileDirectory                       *
//  This is the directory of the current profile.          *
//  Example: /home/userid/FireFoxProfiles/HotLine          *
//**********************************************************

function T_ProfileDirectory(FileName){
  this.Initialize(FileName);
}// of T_ProfileDirectory

//  Inherit from T_Path
T_ProfileDirectory.prototype=new T_Path(T_Path.CURRENT_PROFILE_DIRECTORYV);
T_ProfileDirectory.prototype.constructor=T_ProfileDirectory;

//**********************************************************
//                T_DefaultProfilesDirectory               *
//  This is the directory in which default user profiles   *
//  are created. A default user profile directory is       *
//  created if a specific folder is not selected when      *
//  creating a new profile with ProfileManager.            *
//  Example: /home/userid/.mozilla/firefox                 *
//**********************************************************

function T_DefaultProfilesDirectory(FileName){
  this.Initialize(FileName);
}// of T_DefaultProfilesDirectory

//  Inherit from T_Path
T_DefaultProfilesDirectory.prototype=new T_Path(T_Path.DEFAULT_PROFILES_DIRECTORYV);
T_DefaultProfilesDirectory.prototype.constructor=T_DefaultProfilesDirectory;

//**********************************************************
//                T_ChromeDirectory                        *
//  This is the chrome directory.                          *
//  Example: /home/userid/FireFoxProfiles/HotLine/chrome   *
//**********************************************************

function T_ChromeDirectory(FileName){
  this.Initialize(FileName);
}// of T_ChromeDirectory

//  Inherit from T_Path
T_ChromeDirectory.prototype=new T_Path(T_Path.CHROME_DIRECTORYV);
T_ChromeDirectory.prototype.constructor=T_ChromeDirectory;

//**********************************************************
//                T_TempDirectory                          *
//  This is the system temporary directory.                *
//  Example: /tmp                                          *
//**********************************************************

function T_TempDirectory(FileName){
  this.Initialize(FileName);
}// of T_TempDirectory

//  Inherit from T_Path
T_TempDirectory.prototype=new T_Path(T_Path.TEMP_DIRECTORYV);
T_TempDirectory.prototype.constructor=T_TempDirectory;

//**********************************************************
//                T_HomeDirectory                          *
//  This is the user's home directory                      *
//  Example: /home/userid                                  *
//**********************************************************

function T_HomeDirectory(FileName){
  this.Initialize(FileName);
}// of T_HomeDirectory

//  Inherit from T_Path
T_HomeDirectory.prototype=new T_Path(T_Path.HOME_DIRECTORYV);
T_HomeDirectory.prototype.constructor=T_HomeDirectory;

//**********************************************************
//                T_InstallationDirectory                  *
//  This is the FireFox Installation directory             *
//  Example: /usr/local/firefox                            *
//**********************************************************

function T_InstallationDirectory(FileName){
  this.Initialize(FileName);
}// of T_InstallationDirectory

//  Inherit from T_Path
T_InstallationDirectory.prototype=new T_Path(T_Path.INSTALLATION_DIRECTORYV);
T_InstallationDirectory.prototype.constructor=T_InstallationDirectory;

//**********************************************************
//                T_PreferencesDirectory                   *
//  This is the FireFox Preferences directory              *
//  Example: /home/userid/FireFoxProfiles/HotLine          *
//**********************************************************

function T_PreferencesDirectory(FileName){
  this.Initialize(FileName);
}// of T_PreferencesDirectory

//  Inherit from T_Path
T_PreferencesDirectory.prototype=new T_Path(T_Path.PREFERENCES_DIRECTORYV);
T_PreferencesDirectory.prototype.constructor=T_PreferencesDirectory;

//**********************************************************
//                T_PreferencesFile                        *
//  This is the FireFox Preferences File.                  *
//  Example: /home/userid/FireFoxProfiles/HotLine/prefs.js *
//**********************************************************

function T_PreferencesFile(){
  this.Initialize();
}// of T_PreferencesFile

//  Inherit from T_Path
T_PreferencesFile.prototype=new T_Path(T_Path.PREFERENCES_FILEV);
T_PreferencesFile.prototype.constructor=T_PreferencesFile;

//**********************************************************
//                T_ExtensionDirectory                     *
//  This is the FireFox Extension Directory for the        *
//  given ExtensionName.                                   *
//  Example: /home/userid/hotline                          *
//  If Directories are supplied, they are appended to      *
//  the Extension Directory. Each Directory must be a      *
//  single level name.                                     *
//  Example: /home/userid/hotline/locale/en-AU             *
//  The ExtensionName is the ExtensionID, such as          *
//      hotline@www.metalogic.eu.com                       *
//**********************************************************

function T_ExtensionDirectory(ExtensionName,Directories___){
  //  Variable number of Directory Arguments
  this.Initialize(ExtensionName);
  if ((this.IsValid) && ((nArguments=arguments.length) > 1))
    for (var iArgument=1;iArgument<nArguments;iArgument++)
      this.nsIFile.append(arguments[iArgument]);
}// of T_ExtensionDirectory

//  Inherit from T_Path
T_ExtensionDirectory.prototype=new T_Path(T_Path.EXTENSION_DIRECTORYV);
T_ExtensionDirectory.prototype.constructor=T_ExtensionDirectory;

T_ExtensionDirectory.prototype.Initialize = function (ExtensionName) {
  try {
    this.nsIFile = Components.classes["@mozilla.org/extensions/manager;1"]
                                          .getService(Components.interfaces.nsIExtensionManager)
                                                 .getInstallLocation(ExtensionName)
                                                        .getItemLocation(ExtensionName); 
  }
  catch(e) {
    this.nsIFile = undefined;
  };
  this.IsValid = (this.nsIFile != undefined);
//  return (this.IsValid ? Error(e) : OK());
}// of T_ExtensionDirectory.prototype.Initialize

//************************************************************************
//                T_LocalDirectory                                       *
//  This is a local file, specified as a FileTitle with optional         *
//  extra FileIds. If FileIds are supplied, the FileTitle may be         *
//  either a FileTitle or a Directory. Each FileId must be a single      *
//  level name.                                                          *
//  Example:                                                             *
//   var Path=new T_LocalDirectory('C:\\\\temp','test1.txt');            *
//   if (Path.IsValid){                                                  *
//     var File=new T_File(Path);                                        *
//     if (!File.Open(T_File.ReadOnly,false,Reason))                     *
//               ....                                                    *
//   }                                                                   *
//************************************************************************

function T_LocalDirectory(FileTitle,ExtraFileIds___){
  //  Note: When cloning T_LocalDirectory, the FileTitle is null, but
  //        then nsIFile is cloned and IsValid is set.
  if (FileTitle){
    this.Initialize(FileTitle);
    if (this.IsValid){
      var nArguments=arguments.length;
      if (nArguments > 1)
        for (var iArgument=1;iArgument<nArguments;iArgument++)
          this.nsIFile.append(arguments[iArgument]);
    };
  };
}// of T_LocalDirectory

//  Inherit from T_Path
T_LocalDirectory.prototype=new T_Path(null);
T_LocalDirectory.prototype.constructor=T_LocalDirectory;

T_LocalDirectory.prototype.Initialize = function (FileTitle) {
  try {
    this.nsIFile = Components.classes["@mozilla.org/file/local;1"].createInstance().
                           QueryInterface(Components.interfaces.nsILocalFile);
  }
  catch(e) {
    this.nsIFile = undefined;
  };
  if (this.IsValid = (this.nsIFile != undefined))
    this.nsIFile.initWithPath(FileTitle);
//  return (this.IsValid ? Error(e) : OK());
}// of T_LocalDirectory.prototype.Initialize

//************************************************************************
//   NOTE: THIS IS BEING REPLACED BY T_LocalPath ABOVE.                  *
//                T_LocalFile                                            *
//  This is a local file, specified as a FileTitle with optional         *
//  extra FileIds. If FileIds are supplied, the FileTitle may be         *
//  either a FileTitle or a Directory. Each FileId must be a single      *
//  level name.                                                          *
//  Example:                                                             *
//   var Path=new T_LocalFile('C:\\\\temp','test1.txt');                 *
//   if (Path.IsValid){                                                  *
//     var File=new T_File(Path);                                        *
//     if (!File.Open(T_File.ReadOnly,false,Reason))                     *
//               ....                                                    *
//   }                                                                   *
//************************************************************************

function T_LocalFile(FileTitle,ExtraFileIds___){
  //  Note: When cloning T_LocalFile, the FileTitle is null, but
  //        then nsIFile is cloned and IsValid is set.
  if (FileTitle){
    this.Initialize(FileTitle);
    var nArguments=arguments.length;
    if ((this.IsValid) && (nArguments > 1))
      for (var iArgument=1;iArgument<nArguments;iArgument++)
        this.nsIFile.append(arguments[iArgument]);
  };
}// of T_LocalFile

//  Inherit from T_Path
T_LocalFile.prototype=new T_Path(null);
T_LocalFile.prototype.constructor=T_LocalFile;

T_LocalFile.prototype.Initialize = function (FileTitle) {
  try {
    this.nsIFile = Components.classes["@mozilla.org/file/local;1"].createInstance().
                           QueryInterface(Components.interfaces.nsILocalFile);
  }
  catch(e) {
    this.nsIFile = undefined;
  };
  if (this.IsValid = (this.nsIFile != undefined))
    this.nsIFile.initWithPath(FileTitle);
//  return (this.IsValid ? Error(e) : OK());
}// of T_LocalFile.prototype.Initialize


//**********************************************************
//                T_File                                   *
//  When the T_File is created, a T_Path object should     *
//  be provided. The T_Path object should provide the      *
//  fully qualified FileTitle of the File.                 *
//  Example:                                               *
//    var F = new T_File(                                  *
//              new T_ExtensionDirectory(                  *
//                "hotline@www.metalogic.eu.com",          *
//                "content",                               *
//                "hosts.rdf"));                           *
//    alert("Path =" +F.Path.ToDisplay());                 *
//**********************************************************

//  T_File Class and Properties

function T_File(Path){
  this.Path = Path;
  this.nsIFile = this.Path.nsIFile;
  this.EOLN='\n';
}// of T_File

//  File Use

T_File.ReadOnly             = 0x01;
T_File.ReadAndWrite         = 0x04;

//  Mozilla FLags
T_File.PR_RDONLY            = 0x01;
T_File.PR_WRONLY            = 0x02;
T_File.PR_RDWR              = 0x04;
T_File.PR_CREATE_FILE       = 0x08;
T_File.PR_APPEND            = 0x10;
T_File.PR_TRUNCATE          = 0x20;
T_File.PR_SYNC              = 0x40;
T_File.PR_EXCL              = 0x80;

//  T_File Methods

T_File.prototype.AvailableDiskSpace= function(){
}// of T_File.prototype.AvailableDiskSpace

T_File.prototype.FileName = function(){
  return this.Path.ToDisplay();
}// of T_File.prototype.FileName

T_File.prototype.Resident = function(){
  return this.nsIFile.exists();
}// of T_File.prototype.Resident

T_File.prototype.Open = function(FileUse,NewFile,Reason){
  //  Prepares the File for doing IO.
  //  If NewFile is true, a new file is
  //  created. If the file already exists,
  //  it is truncated to zero.
  var Flags;

  //  Check it is a File
  if (this.nsIFile.exists())
    if (!this.nsIFile.isFile()){
      Reason.Text=this.FileName + ' is Not a File';
      return false;
    };
  //  Setup the DataStream
  this.Reader=(FileUse == T_File.ReadOnly) || (FileUse == T_File.ReadAndWrite);
  this.Writer=(FileUse == T_File.ReadAndWrite);
  //    Writer
  if (this.Writer || NewFile){
    //  Create the Output Stream
   this.OutputStream=Components.classes["@mozilla.org/network/file-output-stream;1"]
                                          .createInstance(Components.interfaces.nsIFileOutputStream);
   Flags=0x02 | 0x10;     //  PR_RDWR | PR_APPEND
   if (NewFile)
     Flags|=0x08 | 0x20;  //  PR_CREATE_FILE | PR_TRUNCATE
   this.OutputStream.init(this.nsIFile,Flags,0664,0);
  };
  //    Reader
  if (this.Reader){
    this.EOF=false;
     //  Create the Input Stream
    this.InputStream=Components.classes["@mozilla.org/network/file-input-stream;1"]
                                           .createInstance(Components.interfaces.nsIFileInputStream);
    //  Simple I/O Stream
    this.ScriptableStream=Components.classes["@mozilla.org/scriptableinputstream;1"]
                                           .createInstance(Components.interfaces.nsIScriptableInputStream);
    //  ReadLn Stream
    this.InputStream.init(this.nsIFile,1,0,false);
    this.ScriptableStream.init(this.InputStream);
    this.InputStream.QueryInterface(Components.interfaces.nsILineInputStream);
  };
  return true;
}// of T_File.prototype.Open

T_File.prototype.ReadLn = function(Data,Reason){
  //  Read the next logical line into Data, which is
  //  a T_ByReference variable.
  //  The function returns true if the newt line has been
  //  Read, otherwise false and a Reason.
  //  Note:  Use ReadLn to read logical records, or 
  //         Read to read bytes, but mixing them has 
  //         not been tested.
  if (!this.Reader)
    return false;
  if (!this.EOF){
    var buffer={};
    var MoreData=this.InputStream.readLine(buffer);
    Data.Value=buffer.value;
    this.EOF=!MoreData;
    return true;
  };
  Reason.Text='EOF';
  return false;
}// of T_File.prototype.ReadLn

T_File.prototype.Read = function(Data,nBytes,Reason){
  //  Read nBytes from the File
  //  If nBytes is -1, then all available bytes
  //  are returned.
  //  Note:  Use ReadLn to read logical records, or 
  //         Read to read bytes, but mixing them has 
  //         not been tested.
  this.EOF=(this.ScriptableStream.available() == 0);
  if (!this.EOF){
    var buffer=this.ScriptableStream.read(nBytes);
    Data.Value=buffer;
    if (Data.Value.length==0){
      Reason.Text='Zero Length. Requested ' + nBytes + 
                  ' bytes, but Read 0 Bytes. ' +
                  this.ScriptableStream.available() + 
                  ' bytes available';
      return false;
    };
    return true;
  };
  Reason.Text='EOF';
  return false;
}// of T_File.prototype.Read

T_File.prototype.Write = function(Data,Reason){
  //  Write Data, which is a String.
  if (this.Writer){
    var IOBytes=this.OutputStream.write(Data,Data.length);
    if (IOBytes != Data.length){
      Reason.Text='Short Block. Requested ' + Data.length + ' bytes, but ' +
                  IOBytes + ' bytes were written';
      return false;
    };
  };
  return true;
}// of T_File.prototype.Write

T_File.prototype.WriteLn = function(Data,Reason){
  //  Write Data and Append this.EOLN
  return this.Write(Data + this.EOLN,Reason);
}// of T_File.prototype.WriteLn

T_File.prototype.Close = function(Reason){
  //  Close the Streams
  if (this.Writer)
    this.OutputStream.close();
  if (this.Reader){
    this.InputStream.close();
    this.ScriptableStream.close();
  };
  return true;
}// of T_File.prototype.Close

T_File.prototype.Purge = function(Reason){
  this.nsIFile.remove(false);
  return true;
}// of T_File.prototype.Purge

T_File.prototype.CopyTo = function(Directory,Reason){
  //  Copies this file to the location specified
  //  by the T_Directory Object
  try{
    this.nsIFile.copyTo(Directory.Path.nsIFile,null);
  }
  catch(e){
    Reason.Text=e.message;
    return false;
  };
  return true;
}// of T_File.prototype.CopyTo

T_File.prototype.CopyAs = function(Directory,FileName,Reason){
  //  Copies this file to the location specified
  //  by the T_Directory Object, with the FileName 
  //  supplied.
  try{
    this.nsIFile.copyTo(Directory.Path.nsIFile,FileName);
  }
  catch(e){
    Reason.Text='CopyAs ' + Directory.Name() + ' -> ' +  FileName + ':' + e.message;
    return false;
  };
  return true;
}// of T_File.prototype.CopyAs


T_File.prototype.MoveTo = function(Directory,Reason){
  //  Moves this file to the location specified
  //  by the T_Directory Object
  try{
    this.nsIFile.moveTo(Directory.Path.nsIFile,null);
  }
  catch(e){
    Reason.Text=e.message;
    return false;
  };
  return true;
}// of T_File.prototype.MoveTo

T_File.prototype.Rename = function(FileName,Reason){
  //  Renames this File
  try{
    this.nsIFile.moveTo(null,FileName);
  }
  catch(e){
    Reason.Text='Rename ' + this.Path.ToDisplay() + ' to ' + FileName + ':' + e.message;
    return false;
  };
  return true;
}// of T_File.prototype.Rename


//**********************************************************
//                T_Directory                              *
//  When the T_Directory is created, a T_Path object       *
//  should be provided. The T_Path object provides the     *
//  fully qualified Directory.                             *
//  Example:                                               *
//    var Dir = new T_Directory(                           *
//                   new T_ProfileDirectory("hotline"));   *
//    if (Dir.Make(0777,Reason))                           *
//      alert("Directory =" +Dir.Name());                  *
//**********************************************************

//  T_Directory Class and Properties

function T_Directory(Path){
  this.Path = Path;
  this.nsIFile = this.Path.nsIFile;
}// of T_Directory

//  T_Directory Methods

T_Directory.prototype.Name = function(){
  return this.Path.ToDisplay();
}// of T_Directory.prototype.Name

T_Directory.prototype.Resident = function(){
  return this.nsIFile.exists();
}// of T_Directory.prototype.Resident

T_Directory.prototype.Make = function(Permissions,Reason){
  //  Makes the Directory specified by the Path.
  //  If the Directory already exists, or is 
  //  created, the funtion returns True, otherwise
  //  False and a Reason.

  //  Check it is a Directory
  if (this.nsIFile.exists()){
    if (!this.nsIFile.isDirectory()){
      Reason.Text=this.FileName + ' exists but is not a directory';
      return false;
    };
    return true;
  }
  //  Create a new Directory
  try{
    this.nsIFile.create(T_Path.DIRECTORY_TYPEV,Permissions)
  }
  catch(e){
    Reason.Text=e.message;
    return false;
  };
  return true;
}// of T_Directory.prototype.Make

T_Directory.prototype.Rename = function(DirectoryName,Reason){
  //  Renames the Directory
  try{
    this.nsIFile.moveTo(null,DirectoryName);
  }
  catch(e){
    Reason.Text=e.message;
    return false;
  };
  return true;
}// of T_Directory.prototype.Rename

T_Directory.prototype.Remove = function(Reason){
  //  Removes this Directory and all Files
  try{
    this.nsIFile.remove(true);
  }
  catch(e){
    Reason.Text=e.message;
    return false;
  };
  return true;
}// of T_Directory.prototype.Remove

//**********************************************************
//                T_ZipFile                                *
//  When the T_ZipFile is created, a T_Path object should  *
//  be provided. The T_path object provides the fully      *
//  qualified Directory.                                   *
//  Example:                                               *
//    var Zip = new T_ZipFile(                             *
//                   new T_LocalFile(MyZipFileTitle));     *
//    if (Zip.                                             *
//**********************************************************

//  T_ZipFile Class and Properties

function T_ZipFile(Path){
  this.Path = Path;
  this.nsIFile = this.Path.nsIFile;
  this.nsIZipWriter = Components.Constructor("@mozilla.org/zipwriter;1","nsIZipWriter");
  this.nsIZipReader = Components.Constructor("@mozilla.org/libjar/zip-reader;1","nsIZipReader");
}// of T_ZipFile

//  T_ZipFile Methods

T_ZipFile.prototype.FileName = function(){
  return this.Path.ToDisplay();
}// of T_ZipFile.prototype.FileName

T_ZipFile.prototype.Resident = function(){
  return this.nsIFile.exists();
}// of T_ZipFile.prototype.Resident

T_ZipFile.prototype.Comment = function(){
  //  Cannot get the Comment from ZipReader
  return (this.Writer?this.ZipWriter.comment:'');
}// of T_ZipFile.prototype.Comment

T_ZipFile.prototype.SetComment = function(Comment){
  if (this.Writer)
    this.ZipWriter.comment=Comment;
}// of T_ZipFile.prototype.SetComment

T_ZipFile.prototype.Open = function(FileUse,NewFile,Reason){
  //  Prepares the File for doing IO.
  //  If NewFile is true, a new file is
  //  created. If the file already exists,
  //  it is truncated to zero.
  var Flags;

  //  Check it is a File
  if (this.nsIFile.exists())
    if (!this.nsIFile.isFile()){
      Reason.Text=this.FileName + ' is Not a File';
      return false;
    };
  //  Setup the DataStream
  this.Reader=(!NewFile) && ((FileUse == T_File.ReadOnly) || (FileUse == T_File.ReadAndWrite));
  this.Writer=(FileUse == T_File.ReadAndWrite);
  //    Writer
  if (this.Writer || NewFile){
     //  Create the ZipFile
    this.ZipWriter=new this.nsIZipWriter();
    Flags= T_File.PR_RDWR;
    if (NewFile)
      Flags|= T_File.PR_CREATE_FILE | T_File.PR_TRUNCATE;
    try{
      this.ZipWriter.open(this.nsIFile,Flags);
    }
    catch(e){
      Reason.Text=e.message;
      return false;
    };
  };
  //    Reader
  if (this.Reader){
     //  Open the ZipFile
    this.ZipReader=new this.nsIZipReader();
    try{
      this.ZipReader.open(this.nsIFile);
    }
    catch(e){
      Reason.Text=e.message;
      return false;
    };
  };
  return true;
}// of T_ZipFile.prototype.Open

T_ZipFile.prototype.AddFile = function(FileName,T_File,OverWrite,Reason){
  if (!this.Writer){
    Reason.Text='Not Writeable';
    return false;
  };
   try{
     if (this.ZipWriter.hasEntry(FileName))
       if (OverWrite)
         this.ZipWriter.removeEntry(FileName,false);
       else{
         Reason.Text='AddFile ' + FileName + ':File Already Exists';
         return false;
       };
     this.ZipWriter.addEntryFile(FileName,
                                 Components.interfaces.nsIZipWriter.COMPRESSION_DEFAULT,
                                 T_File.nsIFile,
                                 false);  // Synchronous Add
   }
   catch(e){
     Reason.Text='AddFile ' + FileName + ':' + e.message;
     return false;
   };
   return true;
}// of T_ZipFile.prototype.AddFile

T_ZipFile.prototype.AddText = function(FileName,Text,OverWrite,Reason){
  if (!this.Writer){
    Reason.Text='Not Writeable';
    return false;
  };
   try{
     if (this.ZipWriter.hasEntry(FileName))
       if (OverWrite)
         this.ZipWriter.removeEntry(FileName,false);
       else{
         Reason.Text='AddFile ' + FileName + ':File Already Exists';
         return false;
       };
     var InputStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
                                     .createInstance(Components.interfaces.nsIStringInputStream);
     InputStream.setData(Text,Text.length);
     this.ZipWriter.addEntryStream(FileName,
                                   (new Date()).getTime()*1000,  // PRTime is microseconds
                                   Components.interfaces.nsIZipWriter.COMPRESSION_DEFAULT,
                                   InputStream,
                                   false);  // Synchronous Add
   }
   catch(e){
     Reason.Text='AddText ' + FileName + ':' + e.message;
     return false;
   };
   return true;
}// of T_ZipFile.prototype.AddText

T_ZipFile.prototype.ReadText = function(FileName,Reason){
  if (!this.Reader){
    Reason.Text='Not Readable';
    return null;
  };
   try{
     var ScriptableInputStream =
              Components.classes["@mozilla.org/scriptableinputstream;1"]
                                  .createInstance(Components.interfaces.nsIScriptableInputStream);
     ScriptableInputStream.init(this.ZipReader.getInputStream(FileName));
     var Text='';
     do {
       var Data=ScriptableInputStream.read(1024);
       Text+=Data;
     } while (Data.length > 0);
   }
   catch(e){
     Reason.Text='ReadText ' + FileName + ':' + e.message;
     return null;
   };
   return Text;
}// of T_ZipFile.prototype.ReadText

T_ZipFile.prototype.RemoveFile = function(FileName,Reason){
  if (!this.Writer){
    Reason.Text='Not Writeable';
    return false;
  };
   try{
     this.ZipWriter.removeEntry(FileName,false);  // Synchronous Remove
   }
   catch(e){
     Reason.Text='RemoveFile ' + FileName + ':' + e.message;
     return false;
   };
   return true;
}// of T_ZipFile.prototype.RemoveFile

T_ZipFile.prototype.Extract = function(FileName,T_LocalFile,Reason){
  if (!this.Reader){
    Reason.Text='Not Readable';
    return false;
  };
   try{
     this.ZipReader.extract(FileName,T_LocalFile.nsIFile);
   }
   catch(e){
     Reason.Text='Extract ' + FileName + ' from ' + T_LocalFile.ToDisplay() + ' :' + e.message;
     return false;
   };
   return true;
}// of T_ZipFile.prototype.Extract

T_ZipFile.prototype.Close = function(Reason){
  //  Close the Zip Files
  if (this.Writer)
    this.ZipWriter.close();
  if (this.Reader)
    this.ZipReader.close();
  return true;
}// of T_ZipFile.prototype.Close

T_ZipFile.prototype.Purge = function(Reason){
  this.nsIFile.remove(false);
  return true;
}// of T_ZipFile.prototype.Purge

T_ZipFile.prototype.CopyTo = function(Directory,FileName,Reason){
  //  Copies this file to the location specified
  //  by the T_Directory Object, with the FileName specified or
  //  if null, the current FileName
  try{
    this.nsIFile.copyTo(Directory.Path.nsIFile,FileName);
  }
  catch(e){
    Reason.Text='CopyTo to ' + Directory.Path.ToDisplay() + ':' + e.message;
    return false;
  };
  return true;
}// of T_ZipFile.prototype.CopyTo

T_ZipFile.prototype.CopyAs = function(Directory,FileName,Reason){
  //  Copies this file to the location specified
  //  by the T_Directory Object, with the FileName 
  //  supplied.
  try{
    this.nsIFile.copyTo(Directory.Path.nsIFile,FileName);
  }
  catch(e){
    Reason.Text='CopyAs ' + Directory.Name() + ' -> ' +  FileName + ':' + e.message;
    return false;
  };
  return true;
}// of T_ZipFile.prototype.CopyAs

T_ZipFile.prototype.MoveTo = function(Directory,Reason){
  //  Moves this file to the location specified
  //  by the T_Directory Object
  try{
    this.nsIFile.moveTo(Directory.Path.nsIFile,null);
  }
  catch(e){
    Reason.Text='Move to ' + Directory.Path.ToDisplay() + ':' + e.message;
    return false;
  };
  return true;
}// of T_ZipFile.prototype.MoveTo

T_ZipFile.prototype.Rename = function(FileName,Reason){
  //  Renames this File
  try{
    this.nsIFile.moveTo(null,FileName);
  }
  catch(e){
    Reason.Text='Rename ' + FileName + ':' + e.message;
    return false;
  };
  return true;
}// of T_ZipFile.prototype.Rename

