/***********************************************************
 *                   RDFSupport                            *
 * This file contains JavaScript types and functions for   *
 * use with RDF/XML files.                                 *
 *                                                         *
 * XULPLanet References:                                   *
 *   Introduction to the RDF Model                         *
 *   RDF DataSources                                       *
 *   Modifying RDF DataSources                             *
 *   Saving RDF                                            *
 *                                                         *
 * Uses                                                    *
 *   filemanager.js                                        *
 *                                                         *
 *                      History                            *
 * 08/03/2006  Initial Version                             *
 * 23/05/2006  HotLine Version 4.0                         *
 * 19/06/2006  GetLiteral accepts nsIRDFRessource or String*
 ***********************************************************/

//**********************************************************
//                T_RDF                                    *
//  This type represents an RDF DataSource at a URI.       *
//**********************************************************

//  T_RDF Class and Properties

function T_RDF(NameSpace){
  this.NameSpace = NameSpace;
  this.RDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"].
                                            getService(Components.interfaces.nsIRDFService);
  this.RDFCUtils = Components.classes["@mozilla.org/rdf/container-utils;1"].
                                         getService(Components.interfaces.nsIRDFContainerUtils);
}// of T_RDF

//  T_RDF Methods

T_RDF.prototype.Initialize = function(URI,Wait,Reason){
  //  The URI is the location of the RDF DataSource
  //  to be loaded.
  //  if any error occurs, the function returns false 
  //  a Reason, otherwise true.
  //  Note:  Make sure Wait is TRUE if the DataSource is not loading
  //         and saving properly.
  if (!Wait) 
    alert('Wait is FALSE when loading ' + URI);
  try{
    this.URI=URI;
    if (Wait){
      this.DataSource=this.RDFService.GetDataSourceBlocking(URI);
    }
    else
      this.DataSource=this.RDFService.GetDataSource(URI);
  }
  catch(e){
    Reason.Text='RDF Exception' + (Wait?'( Blocking)':'') +
                ' URI ' + URI + '(' + e.message + ')';
    return false;    
  };
  return true;
}// of T_RDF.prototype.Initialize

T_RDF.prototype.PredicateURI = function(Predicate){
  //  Combine the NameSpace with the Predicate
  //  to give the URI for the Predicate.
  return this.NameSpace + '/' + Predicate;
}// of T_RDF.prototype.PredicateURI

T_RDF.prototype.SubjectURI = function(Subject){
  //  Combine the NameSpace with the Subject
  //  to give the URI for the Subject.
  return this.NameSpace + '/' + Subject;
}// of T_RDF.prototype.SubjectURI

T_RDF.prototype.Triple = function(Subject,Predicate,Object){
  //  Display the Strings in Subject,Predicate
  //  and Object as a Triple
  return '(' + Subject + '=>' + Predicate + '=>' + Object + ')';
}// of T_RDF.prototype.Triple


T_RDF.prototype.Save = function(){
  //  Save any changes made to the DataSource
  //  back to the original URI.
  var DataSource = this.DataSource;
  DataSource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
  DataSource.Flush();
}// of T_RDF.prototype.Save

T_RDF.prototype.GetLiteral = function(SubjectURI,PredicateURI,Literal,Reason){
  //  Given the Subject and Predicate, return the Literal.
  //  If it does not exist, return FALSE and a Reason,
  //  otherwise TRUE. Literal is T_ByReference.
  //  SubjectURI may be either a String or an nsIRDFResource.
  if (typeof SubjectURI == 'string')
    var Subject=this.RDFService.GetResource(SubjectURI)
  else
    var Subject=SubjectURI;
  var Predicate=this.RDFService.GetResource(PredicateURI);
  var Target;
  Target=this.DataSource.GetTarget(Subject,Predicate,true);
  if (Target==null){
    Reason.Text='Not Found ' + this.Triple(SubjectURI,PredicateURI,'');
    return false;
  };
  if (Target instanceof Components.interfaces.nsIRDFLiteral)
    Literal.Value=Target.Value
  else
    Literal.Value=Target;
  return true;
}// of T_RDF.prototype.GetLiteral

T_RDF.prototype.SetLiteral = function(SubjectURI,PredicateURI,Literal,Reason){
  //  Create the Triple given by the Subject, Predicate and Literal.
  //  If the Subject and Predicate already exist, the Object is 
  //  changed to be the Literal. If any error occurs, the function
  //  returns FALSE and a Reason.
  var RDFSubject=this.RDFService.GetResource(SubjectURI);
  var RDFPredicate=this.RDFService.GetResource(PredicateURI);
  var RDFLiteral=this.RDFService.GetLiteral(Literal)
  var Target;

  Target=this.DataSource.GetTarget(RDFSubject,RDFPredicate,true);
 //alert(SubjectURI+','+PredicateURI+','+Target+',Lit='+Literal+',val='+Target.Value);
  if (Target!=null){
    if (Target instanceof Components.interfaces.nsIRDFLiteral){
      if (Target.Value == Literal) 
         return true;
      //  Modify the Literal
      this.DataSource.Change(RDFSubject,RDFPredicate,Target,RDFLiteral);
      this.Save();
      return true;
    }
    else{
      Reason.Text='Not Literal ' + this.Triple(SubjectURI,PredicateURI,'');
      return;
    };
  };
  this.DataSource.Assert(RDFSubject,RDFPredicate,RDFLiteral,true);
  this.Save();
  return true;
}// of T_RDF.prototype.SetLiteral

T_RDF.prototype.DeleteLiteral = function(SubjectURI,PredicateURI,Reason){
  //  Delete the triple with the Subject and Predicate 
  //  providing that the Object is a Literal.
  //  If the triple is deleted, the function returns TRUE,
  //  otherwise FALSE and a Reason.
  var Subject=this.RDFService.GetResource(SubjectURI);
  var Predicate=this.RDFService.GetResource(PredicateURI);
  var Target;
  Target=this.DataSource.GetTarget(Subject,Predicate,true);
  if (Target==null){
    Reason.Text='Not Found ' + this.Triple(SubjectURI,PredicateURI,'');
    return false;
  };
  if (!Target instanceof Components.interfaces.nsIRDFLiteral){
     Reason.Text='Not a Literal ' + this.Triple(SubjectURI,PredicateURI,'');
     return false;
  };
  this.DataSource.Unassert(Subject,Predicate,Target);
  return true;
}// of T_RDF.prototype.DeleteLiteral

T_RDF.prototype.AddToDataSources = function (ElementName,Rebuild){
  //  Add the configuration file DataSource to 
  //  the Element with the given name. If Rebuild is
  //  set, then rebuild the user interface.
  var ElementRef=document.getElementById(ElementName);
  var DataSources=ElementRef.database;
  try{
    DataSources.AddDataSource(this.DataSource);
  }catch(e){
    alert('Element ' + ElementName + ':' + e.message); 
  };
  if (Rebuild){
    ElementRef.builder.rebuild();
  };
}// of T_RDF.prototype.AddToDataSources

T_RDF.prototype.RemoveFromDataSources = function (ElementName,Rebuild){
  //  Remove the configuration file DataSource from 
  //  the Element with the given name. If Rebuild is
  //  set, then rebuild the user interface.
  var ElementRef=document.getElementById(ElementName);
  var DataSources=ElementRef.database;
  DataSources.RemoveDataSource(this.DataSource);
  if (Rebuild){
    ElementRef.builder.rebuild();
  };
}// of T_RDF.prototype.RemoveFromDataSources

T_RDF.prototype.ResetDataSources = function (ElementName,Rebuild){
  //  Reset the DataSources for the Element with the
  //  given name to reference only this configuration
  //  file DataSource. If Rebuild is set, then 
  //  rebuild the user interface.
  var ElementRef=document.getElementById(ElementName);
  var DataBase=ElementRef.database;
  //  Delete any other DataSources
  var DataSources = DataBase.GetDataSources();
  while (DataSources.hasMoreElements()){
    var ADataSource=DataSources.getNext();
    if (ADataSource instanceof Components.interfaces.nsIRDFDataSource){
      DataBase.RemoveDataSource(ADataSource);
    };
  };
  //  Add this DataSource
  DataBase.AddDataSource(this.DataSource);
  if (Rebuild){
    ElementRef.builder.rebuild();
  };
}// of T_RDF.prototype.ResetDataSources


T_RDF.TimeStamp = function(){
  return FormatYYYYMMDDHHMMSSNNN(YYYYMMDDHHMMSSNNN(new Date()));
}// of T_RDF.TimeStamp

//**********************************************************
//                T_RDFContainer                           *
//  This type represents a Seq Container.                  *
//**********************************************************

//  T_RDFContainer Class and Properties

function T_RDFContainer(Parent,About){
  //  Create the T_RDFContainer with a Reference 
  //  to the Parent T_RDF object and initialize
  //  a Node for the About String.
  this.Parent = Parent;
  this.Container = Components.classes["@mozilla.org/rdf/container;1"].
                                             createInstance(Components.interfaces.nsIRDFContainer);
  this.About=About;
  this.Node = this.Parent.RDFService.GetResource(About);
}// of T_RDFContainer

//  T_RDFContainer Methods

T_RDFContainer.prototype.IsContainer = function(Reason){
  //  The function returns TRUE if the Container
  //  already exists, otherwise FALSE and a Reason.
  var Result=this.Parent.RDFCUtils.IsContainer(this.Parent.DataSource,this.Node);
  if (!Result)
    Reason.Text=this.About + ' is Not Present or is Not a Container';
  return Result;
}// of T_RDFContainer.prototype.IsContainer

T_RDFContainer.prototype.Create = function(Reason){
  //  Create a Seq Container specified by the About String
  this.Parent.RDFCUtils.MakeSeq(this.Parent.DataSource,this.Node);
  this.Container.Init(this.Parent.DataSource,this.Node);
  return true;
}// of T_RDFContainer.prototype.Create

T_RDFContainer.prototype.Destroy = function(Reason){
  //  Destroy this Seq Container and any list items
  //  Reference: http://mozdev.org/pipermail/
  //                      project_owners/2005-May/004569.html
  var arcOut=this.Parent.DataSource.ArcLabelsOut(this.Node);
  while (arcOut.hasMoreElements()){
     var aLabel=arcOut.getNext();
     if (aLabel instanceof Components.interfaces.nsIRDFResource){
       var aTarget=this.Parent.DataSource.GetTarget(this.Node,aLabel,true);
       this.Parent.DataSource.Unassert(this.Node,aLabel,aTarget);
     }
  };
  return true;
}// of T_RDFContainer.prototype.Destroy

T_RDFContainer.prototype.Open = function(Reason){
  //  Open the Seq Container specified by the About String.
  //  If it is opened, the function returns TRUE, otherwise
  //  FALSE and a Reason.
  try{
    this.Container.Init(this.Parent.DataSource,this.Node);
  }
  catch(e){
    Reason.Text='No Container ' + this.About + ' (' + e.message  + ')';
    return false;
    
  };
  return true;
}// of T_RDFContainer.prototype.Open

T_RDFContainer.prototype.Contains = function(Element){
  //  This function returns true if the Element
  //  already exists in the container.
  return (this.Container.IndexOf(Element.Node) >= 0);
}// of T_RDFContainer.prototype.Contains

T_RDFContainer.prototype.Insert = function(Element){
  //  This function adds a list element into this 
  //  Container if it does not already exist.
  //  The list element will contain a Resource
  //  reference to the Element.
  if (this.Container.IndexOf(Element.Node) < 0)
    this.Container.AppendElement(Element.Node);
}// of T_RDFContainer.prototype.Insert

T_RDFContainer.prototype.Append = function(Element){
  //  This function appends a list element to this 
  //  Container. The list element will contain a
  //  Resource reference to the Element.
  this.Container.AppendElement(Element.Node);
}// of T_RDFContainer.prototype.Append

T_RDFContainer.prototype.Remove = function(Element){
  //  This function removes the list element given by
  //  the Element which may be any type with a
  //  a Node property.
  this.Container.RemoveElement(Element.Node,true);
}// of T_RDFContainer.prototype.RemoveDescription

T_RDFContainer.prototype.GetURIList = function(Reason){
  //  This function returns an array containing
  //  the URIs of the arcs which lead out of this
  //  container. Note this includes both RDF
  //  URIs and other nameSpace URIs.
  //  If any error occurs, the function returns 
  //  null and a Reason, otherwise true.
  var URIList=new Array();
  var arcOut=this.Parent.DataSource.ArcLabelsOut(this.Node);
  while (arcOut.hasMoreElements()){
     var aLabel=arcOut.getNext();
     if (aLabel instanceof Components.interfaces.nsIRDFResource){
       var aTarget=this.Parent.DataSource.GetTarget(this.Node,aLabel,true);
       if (aTarget instanceof Components.interfaces.nsIRDFResource)
         URIList.push(aTarget.Value);
     }
  };
  if (URIList.length == 0){
    return null
    Reason.Text='No Elements';
  };
  return URIList;
}// of T_RDFContainer.prototype.GetURIList

//**********************************************************
//                T_RDFDescription                         *
//  This type represents an RDF Description                *
//**********************************************************

//  T_RDFDescription Class and Properties

function T_RDFDescription(Parent){
  //  Create the T_RDFDescription with a Reference 
  //  to the Parent T_RDF object.
  this.Parent = Parent;
}// of T_RDFDescription

//  T_RDFDescription Methods

T_RDFDescription.prototype.Initialize = function(About,Reason){
  //  Creates an RDF Resource for the About string supplied.
  //  If no About string is supplied, an anonymous Resource
  //  is created.
  if (arguments.length==0){
    this.Node = this.Parent.RDFService.GetAnonymousResource();
    this.About=this.Node.Value;
  }
  else {
    this.About=About;
    this.Node = this.Parent.RDFService.GetResource(About);
  };
  return true;
}// of T_RDFDescription.prototype.Initialize

T_RDFDescription.prototype.Insert = function(Attribute){
  //  This function inserts the Predicate and Value element
  //  given by Attribute of type T_RDFAttribute.
  this.Parent.DataSource.Assert(this.Node,Attribute.Predicate,Attribute.Value,true);
}// of T_RDFDescription.prototype.Insert

T_RDFDescription.prototype.Remove = function(Attribute){
  //  This function inserts the Predicate and Value element
  //  given by Attribute of type T_RDFAttribute.
  this.Parent.DataSource.Unassert(this.Node,Attribute.Predicate,Attribute.Value);
}// of T_RDFDescription.prototype.Remove

T_RDFDescription.prototype.Delete = function(Reason){
  //  This function deletes this Resource and all associated
  //  Predicate and Value elements
  var aTarget;
  var arcOut=this.Parent.DataSource.ArcLabelsOut(this.Node);
  while (arcOut.hasMoreElements()){
     var aLabel=arcOut.getNext();
     if (aLabel instanceof Components.interfaces.nsIRDFResource){
       while ((aTarget=this.Parent.DataSource.GetTarget(this.Node,aLabel,true))!=null){
         this.Parent.DataSource.Unassert(this.Node,aLabel,aTarget);
       };
     }
  };
  return true;
}// of T_RDFDescription.prototype.Delete

T_RDFDescription.DeleteNode = function(DataSource,Node,Reason){
  //  This function deletes the Node and all associated
  //  Predicate and Value elements from the DataSource
  var aTarget;
  var arcOut=DataSource.ArcLabelsOut(Node);
  while (arcOut.hasMoreElements()){
     var aLabel=arcOut.getNext();
     if (aLabel instanceof Components.interfaces.nsIRDFResource){
       while ((aTarget=DataSource.GetTarget(Node,aLabel,true))!=null){
         DataSource.Unassert(Node,aLabel,aTarget);
       };
     }
  };
  return true;
}// of  T_RDFDescription.DeleteNode

//**********************************************************
//                T_RDFAttribute                           *
//  This type represents an RDF Attribute. The methods     *
//  allow different types of values to be associated with  *
//  the Attribute. The Attribute may be applied to a       *
//  T_RDFDescription subject.                              *
//**********************************************************

//  T_RDFAttribute Class and Properties

function T_RDFAttribute(Parent,Predicate){
  //  Create the T_RDFAttribute with a Reference 
  //  to the Parent T_RDF object. The Predicate
  //  is an URI which defines the Attribute name.
  this.Parent = Parent;
  this.Predicate = this.Parent.RDFService.GetResource(Predicate);
}// of T_RDFAttribute

//  T_RDFAttribute Methods

T_RDFAttribute.prototype.SetPredicate = function(Predicate){
  //  Creates an RDF Attribute with the given Predicate
  this.Predicate=this.Parent.RDFService.GetResource(Predicate);
}// of T_RDFAttribute.prototype.SetPredicate

T_RDFAttribute.prototype.SetToLiteral = function(Literal){
  //  Sets the Attribute Value to the Literal
  this.Value=this.Parent.RDFService.GetLiteral(Literal)
}// of T_RDFAttribute.prototype.SetToLiteral

T_RDFAttribute.prototype.SetToResource = function(Resource){
  //  Sets the Attribute Value to the Resource
  this.Value=this.Parent.RDFService.GetResource(Resource);
}// of T_RDFAttribute.prototype.SetToResource

T_RDFAttribute.prototype.SetToDateLiteral = function(NumericDate){
  //  Sets the Attribute Value to the Date Literal
  this.Value=this.Parent.RDFService.GetDateLiteral(NumericDate);
}// of T_RDFAttribute.prototype.SetToDateLiteral

T_RDFAttribute.prototype.SetToIntegerLiteral = function(NumericLiteral){
  //  Sets the Attribute Value to the Numeric Literal
  this.Value=this.Parent.RDFService.GetIntLiteral(NumericLiteral);
}// of T_RDFAttribute.prototype.SetToIntegerLiteral

